"""
Selectors for working with albums
"""
from functools import partial
from typing import Iterable

from spotify_model import Paging, SearchAlbum, SimplifiedAlbum, SimplifiedTrack
from spotipy import Spotify

from .util import chunk, exhaust


def album_to_simplified(
    client: Spotify, albums: Iterable[SearchAlbum], chunk_size: int = 20
) -> Iterable[SimplifiedAlbum]:
    "Retrieve the actual album objects associated with the albums received from searching"

    for album_chunk in chunk(albums, chunk_size):
        album_ids = [a.spotify_id for a in album_chunk]

        for album in client.albums(albums=album_ids)["albums"]:
            yield SimplifiedAlbum(**album)


def album_to_tracks(client: Spotify, albums: Iterable[SimplifiedAlbum]) -> Iterable[SimplifiedTrack]:
    "Convert an album stream to the tracks on that album"

    def _album_tracks(album_id: str, limit: int, offset: int) -> Paging:
        return Paging(**client.album_tracks(album_id=album_id, limit=limit, offset=offset))

    # Because most album tracklists don't need to use paging, it's expected that API calls are relatively infrequent
    for album in albums:
        tracks_function = partial(client.album_tracks, album_id=album.spotify_id)

        for track in exhaust(tracks_function, album.tracks):
            yield SimplifiedTrack(**track)
